DEFAULT_INTERNAL_PROTOCOLS = [{
    "name": "SpaceONE User",
    "protocol_type": "INTERNAL",
    "resource_type": "identity.User",
    "capability": {
        'supported_schema': ['spaceone_user']
    },
    "plugin_info": {
        'metadata': {
            'data_type': 'PLAIN_TEXT'
        }
    },
    "tags": {
        "icon": "https://spaceone-custom-assets.s3.ap-northeast-2.amazonaws.com/console-assets/icons/notifications_member.svg"
    }
}]
