PROTO = {
    'spaceone.notification.interface.grpc.v1.protocol': ['Protocol'],
    'spaceone.notification.interface.grpc.v1.quota': ['Quota'],
    'spaceone.notification.interface.grpc.v1.project_channel': ['ProjectChannel'],
    'spaceone.notification.interface.grpc.v1.user_channel': ['UserChannel'],
    'spaceone.notification.interface.grpc.v1.notification': ['Notification'],
    'spaceone.notification.interface.grpc.v1.notification_usage': ['NotificationUsage']
}
